************************************************************************
* File (main_cc_1loop_vegas.F).
************************************************************************
* This is the main FORTRAN module to calculate 1-loop EW and QCD
* radiative correction for the Charged Current Drell-Yan processes.
*
* It is produced by s2n package of SANC (v1.10) Project.
* Copyright (C), SANC Project Team, 2002-2008.
*
* http://sanc.jinr.ru/
* http://pcphsanc.cern.ch/
* E-mail: <sanc@jinr.ru>

************************************************************************
* Table of the processes
* [pid - Process IDentification number]:
* ------------------------------------------------
* | pid | SANC     | Process                     |
* |     | notation |                             |
* ------------------------------------------------
* |  1  | 13141112 | anti-up + dn -> en   + el^- |
* |  2  | 14131211 | anti-dn + up -> el^+ + en   |
* |  3  | 13141516 | anti-up + dn -> mn   + mo^- |
* |  4  | 14131615 | anti-dn + up -> mo^+ + mn   |
* |  5  | 13141920 | anti-up + dn -> tn   + ta^- |
* |  6  | 14132019 | anti-dn + up -> ta^+ + tn   |
* ------------------------------------------------
*
* Types of the 1-loop corrections
* for the processes with pid=1-4:
* ------------------------------------
* | iqed | iew  | iqcd | corrections |
* ------------------------------------
* |  1   |  0   |  0   |   QED       |
* |  0   |  1   |  0   |   Weak      |
* |  0   |  0   |  1   |   QCD       |
* |  1   |  1   |  0   |   EW        |
* |  1   |  0   |  1   |   QED+QCD   |
* |  0   |  1   |  1   |   Weak+QCD  |
* |  1   |  1   |  1   |   EW+QCD    |
* ------------------------------------
************************************************************************
      program main_cc_1loop_vegas
      implicit none!
#include "s2n_declare.h"

      integer*4 ome,lmu,imu,isetup,is,ic,ihard,pid
      integer*4 ncall1,itmx1,itmx2,nrdm
      real*8 sq,virt_ew,virt_qcd,hard_ew,hard_qcd,brdq,brdq_ew,brdq_qcd
      real*8 born_sd,virt_sd,virt_ew_sd,virt_qcd_sd,hard_sd,hard_ew_sd,
     &     hard_qcd_sd,brdq_sd,brdq_ew_sd,brdq_qcd_sd
      real*8 costa(5),sa(8),sres(4,10,8)
      real*8 Mfscale
      common /s/ s
      common /Mfscale/ Mfscale
      common /pid/ pid

      data costa/-0.9d0, -0.5d0, 0d0, +0.5d0, +0.9d0/
      data sa/40d0,80d0,120d0,200d0,500d0,1000d0,2000d0,14000d0/
*
*   iqed     = 0   without QED corrections
*              1   with 
*              2   ISR
*              3   IFI
*              4   FSR
*              5   [IFI+FSR]
*
*   iew      = 0   without Weak corrections
*              1   with
*
*   iqcd     = 0   without QCD corrections
*              1   with
*
*   iborn    = 0   Born + 1-loop
*              1   Born
*
*   gfscheme = 0   alpha(0) calculation scheme
*              1   GF
*              2   GF prime
*                
*   ilin     = 0   without linearization
*              1   with
*
*   ifgg    = -1   ffgg = 0
*              0          zoro
*              1          [zoro+coeff*ff_gg]
*              2          [zoro/(1d0 - coeff*ff_gg)]
*
*   ihard    = 1   1spr integration for hard
*              4   4d integration
*
*   isetup   = 0   SANC input [PDG 2006]            CPC 177 (2007) 623
*              1   Les Houches Workshop (2005)      [hep-ph/0604120]
*              2   Tevatron-for-LHC Workshop (2006) [arXiv:0705.3251]
*              3   Les Houches Workshop (2007)      [arXiv:0803.0678]
*
      iqed = 1
      iew = 1
      iqcd = 0
      iborn = 0
      gfscheme = 0
      ilin = 1
      ifgg = 1
      ihard = 1
      isetup = 1
      pid = 1

      if (isetup.eq.0) then
#include "sanc_input.h"
         print *, "SANC setup"
      elseif (isetup.eq.1) then
#include "leshw_input.h"
         print *, "Les Houches (2005) setup"
      elseif (isetup.eq.2) then
#include "tev4lhcw_input.h"
         print *, "TeV4LHC (2006) setup"
      elseif (isetup.eq.3) then
#include "leshw2007_input.h"
         print *, "Les Houches (2007) setup"
      endif

*      mup = mup/10d0
*      mdn = mdn/10d0

      call s2n_init()
      call ffini()

      call ProcessInit (pid)
      call PrintConsts (0)

      thmu = mw
      tlmu2 = 1d-2
      thmu2 = thmu**2
      print *, ''
      print *, "thmu(GeV)    = ", thmu
      print *, "thla2(GeV^2) = ", tlmu2

      born        = 0d0
      born_sd     = 0d0
      virt        = 0d0
      virt_sd     = 0d0
      virt_ew     = 0d0
      virt_ew_sd  = 0d0
      virt_qcd    = 0d0
      virt_qcd_sd = 0d0
      brdq        = 0d0
      brdq_sd     = 0d0
      brdq_ew     = 0d0
      brdq_ew_sd  = 0d0
      brdq_qcd    = 0d0
      brdq_qcd_sd = 0d0
      hard        = 0d0
      hard_sd     = 0d0
      hard_ew     = 0d0
      hard_ew_sd  = 0d0
      hard_qcd    = 0d0
      hard_qcd_sd = 0d0

      nrdm = 12345

      do is=1,8
         sq = sa(is)
         s  = sq**2

         call BetaInit (pid)
*     born
         iborn = 1
         ncall1 = 100 000
         itmx1 = 10
         itmx2 = 10
         call ProcessBorn
     &        (nrdm,ncall1,itmx1,itmx2,born,born_sd)

         print *, "sq(GeV)        = ", sq
         print *, "[bo](pb)       = ", born, born_sd
         if (iqed.eq.0.and.iew.eq.0.and.iqcd.eq.0) stop

      do ome = 2,3
         if (ome.eq.1) omega = 1d-4*sq/2
         if (ome.eq.2) omega = 1d-5*sq/2
         if (ome.eq.3) omega = 1d-6*sq/2

         print *, ""
         print *, "omega(GeV)     = ", omega

*     EW part
         if ((iew.ne.0).or.(iqed.ne.0)) then
*     virt+soft (EW)
         iborn = 0
         ncall1 = 10 000
         itmx1 = 10
         itmx2 = 10
        call ProcessVirt_EW
     &        (nrdm,ncall1,itmx1,itmx2,virt_ew,virt_ew_sd)
*     brdq (EW)
         ncall1 = 500 000
         itmx1 = 10
         itmx2 = 10
         call ProcessBrdq_EW
     &        (nrdm,ncall1,itmx1,itmx2,brdq_ew,brdq_ew_sd)

         if ((iqed.ge.1).and.(ihard.eq.1)) then
*     hard (EW), 1spr
         ncall1 = 100 000
         itmx1 = 10
         itmx2 = 10
         call ProcessHard_EW_1spr 
     &        (nrdm,ncall1,itmx1,itmx2,hard_ew,hard_ew_sd)

         elseif ((iqed.ge.1).and.(ihard.eq.4)) then
*     hard (EW), 4d
         ncall1 = 1 000 000
         itmx1 = 10
         itmx2 = 10
         call ProcessHard_EW
     &        (nrdm,ncall1,itmx1,itmx2,hard_ew,hard_ew_sd)
         endif
         endif

*     QCD part
         if (iqcd.ne.0) then
     &        then
*     virt+soft (QCD)
         iborn = 0
         ncall1 = 10 000
         itmx1 = 10
         itmx2 = 10
         call ProcessVirt_QCD
     &        (nrdm,ncall1,itmx1,itmx2,virt_qcd,virt_qcd_sd)
*     brdq (QCD)
         iborn = 0
         ncall1 = 10 000
         itmx1 = 10
         itmx2 = 10
         call ProcessBrdq_QCD
     &        (nrdm,ncall1,itmx1,itmx2,brdq_qcd,brdq_qcd_sd)

         if (ihard.eq.1) then
*     hard (QCD), 1spr
         iborn = 0
         ncall1 = 10 000
         itmx1 = 10
         itmx2 = 10
         call ProcessHard_QCD_1spr
     &        (nrdm,ncall1,itmx1,itmx2,hard_qcd,hard_qcd_sd)

         elseif (ihard.eq.4) then
*     hard, 4d
         ncall1 = 1 000 000
         itmx1 = 10
         itmx2 = 10
         call ProcessHard_QCD
     &        (nrdm,ncall1,itmx1,itmx2,hard_qcd,hard_qcd_sd)
         endif
         endif

         virt    = virt_ew + virt_qcd
         virt_sd = dsqrt(virt_ew_sd**2 + virt_qcd_sd**2)
         brdq    = brdq_ew + brdq_qcd
         brdq_sd = dsqrt(brdq_ew_sd**2 + brdq_qcd_sd**2)
         hard    = hard_ew + hard_qcd
         hard_sd = dsqrt(hard_ew_sd**2 + hard_qcd_sd**2)

         print *
         call PrintProcess (pid)
         call PrintConsts (1)
         print *
         print *, "sq(GeV)        = ", sq
         print *, "[bo](pb)       = ", born, born_sd
         print *, "omega(GeV)     = ", omega
         print *, "[vi+so](pb)    = ", virt, virt_sd
         print *, "[ha](pb)       = ", hard, hard_sd
         print *, "[1loop](pb)    = ", born+virt+hard,
     &        sqrt(born_sd**2+virt_sd**2+hard_sd**2)
         print *, "[delta](%)     = ", (virt+hard)/born*100d0,
     &        sqrt(virt_sd**2+hard_sd**2)/born*100d0
         print *, "[brdq](pb)     = ", brdq,brdq_sd
         print *, "[1loop_MS](pb) = ", born+virt+hard-brdq,
     &        sqrt(born_sd**2+virt_sd**2+hard_sd**2+brdq_sd**2)
         print *, "[delta_MS](%)  = ", (virt+hard-brdq)/born*100d0,
     &        sqrt(virt_sd**2+hard_sd**2+brdq_sd**2)/born*100d0

      enddo
      enddo

      stop
      end
************************************************************************
      subroutine ProcessBorn
     &     (nrdm,ncall1,itmx1,itmx2,born,born_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,nrdm
      real*8 born_sd
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      external function_ew_1c

      xl(1) = -1d0
      xu(1) = +1d0

      dummy = ranini(nrdm)
      ncall = ncall1
      itmx = itmx1
      call vegas (1, function_ew_1c, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (1, function_ew_1c, avgi, sd, chi2a)
      born = avgi
      born_sd = sd

      return
      end
************************************************************************
      subroutine ProcessVirt_EW (nrdm,ncall1,itmx1,itmx2,virt,virt_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,nrdm
      real*8 virt_sd,x1cut
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      external function_ew_1c

      x1cut = 1d-15
      xl(1) = -1d0+x1cut
      xu(1) = +1d0-x1cut

      dummy = ranini(nrdm)
      ncall = ncall1
      itmx = itmx1
      call vegas (1, function_ew_1c, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (1, function_ew_1c, avgi, sd, chi2a)
      virt = avgi
      virt_sd = sd

      return
      end
************************************************************************
      subroutine ProcessVirt_QCD (nrdm,ncall1,itmx1,itmx2,virt,virt_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,pid,nrdm
      real*8 virt_sd,x1cut
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      common /pid/ pid
      external function_qcd_1c

      x1cut = 1d-15
      xl(1) = -1d0+x1cut
      xu(1) = +1d0-x1cut

      dummy = ranini(nrdm)
      ncall = ncall1
      itmx = itmx1
      call vegas (1, function_qcd_1c, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (1, function_qcd_1c, avgi, sd, chi2a)
      virt = avgi
      virt_sd = sd

      return
      end
************************************************************************
      subroutine ProcessBrdq_EW (nrdm,ncall1,itmx1,itmx2,brdq,brdq_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,pid,nrdm
      real*8 brdq,brdq_sd
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      real*8 DelSV_MS,Mfscale,sq
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      common /s/ s
      common /Mfscale/ Mfscale
      common /pid/ pid
      external function_ew_1s

      sq = sqrt(s)

      DelSV_MS = 0d0
      brdq     = 0d0
      brdq_sd  = 0d0

      if     (iqed.eq.0) return

      if     (pid.eq.1) then
         call cc_bo_1314_1112 (s,born)
      elseif (pid.eq.2) then
         call cc_bo_1413_1211 (s,born)
      elseif (pid.eq.3) then
         call cc_bo_1314_1516 (s,born)
      elseif (pid.eq.4) then
         call cc_bo_1413_1615 (s,born)
      elseif (pid.eq.5) then
         call cc_bo_1314_1920 (s,born)
      elseif (pid.eq.6) then
         call cc_bo_1413_2019 (s,born)
      endif

*     DelSV_MS (EW)
*     see Brem_4f_CC.frm [#if `SubtScheme' = 1;*** MSbar subtraction]
         DelSV_MS=alpha/2/pi*qup**2*
     & (log(Mfscale**2/mup**2)*log(4d0*omega**2/s)
     &     + 3d0/2*log(Mfscale**2/mup**2) - log(4d0*omega**2/s) - 3d0/2
     &     - 1d0/2*log(4d0*omega**2/s)**2 + 7d0/2)*born
     &           +alpha/2/pi*qdn**2*
     & (log(Mfscale**2/mdn**2)*log(4d0*omega**2/s)
     &     + 3d0/2*log(Mfscale**2/mdn**2) - log(4d0*omega**2/s) - 3d0/2
     &     - 1d0/2*log(4d0*omega**2/s)**2 + 7d0/2)*born
      brdq = DelSV_MS

      xl(1) = mf**2/s
      xu(1) = (s-2d0*sq*omega)/s

      dummy = ranini(nrdm)
      ncall = ncall1
      itmx = itmx1
      call vegas (1, function_ew_1s, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (1, function_ew_1s, avgi, sd, chi2a)
      brdq = brdq + avgi
      brdq_sd = sd

      return
      end
************************************************************************
      subroutine ProcessBrdq_QCD (nrdm,ncall1,itmx1,itmx2,brdq,brdq_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,pid,nrdm
      real*8 brdq,brdq_sd
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      real*8 DelSV_MS,Mfscale,sq
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      common /s/ s
      common /Mfscale/ Mfscale
      common /pid/ pid
      external function_qcd_1s

      sq = sqrt(s)

      DelSV_MS = 0d0
      brdq     = 0d0
      brdq_sd  = 0d0

      if     (pid.eq.1) then
         call cc_bo_1314_1112 (s,born)
      elseif (pid.eq.2) then
         call cc_bo_1413_1211 (s,born)
      elseif (pid.eq.3) then
         call cc_bo_1314_1516 (s,born)
      elseif (pid.eq.4) then
         call cc_bo_1413_1615 (s,born)
      elseif (pid.eq.5) then
         call cc_bo_1314_1920 (s,born)
      elseif (pid.eq.6) then
         call cc_bo_1413_2019 (s,born)
      endif

*     DelSV_MS (QCD)
         DelSV_MS=alphas/2/pi*cf*
     & (log(Mfscale**2/mup**2)*log(4d0*omega**2/s)
     &     + 3d0/2*log(Mfscale**2/mup**2) - log(4d0*omega**2/s) - 3d0/2
     &     - 1d0/2*log(4d0*omega**2/s)**2 + 7d0/2)*born
     &           +alphas/2/pi*cf*
     & (log(Mfscale**2/mdn**2)*log(4d0*omega**2/s)
     &     + 3d0/2*log(Mfscale**2/mdn**2) - log(4d0*omega**2/s) - 3d0/2
     &     - 1d0/2*log(4d0*omega**2/s)**2 + 7d0/2)*born
      brdq = DelSV_MS

      xl(1) = mf**2/s
      xu(1) = (s-2d0*sq*omega)/s

      dummy = ranini(nrdm)
      ncall = ncall1
      itmx = itmx1
      call vegas (1, function_qcd_1s, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (1, function_qcd_1s, avgi, sd, chi2a)
      brdq = brdq + avgi
      brdq_sd = sd

      return
      end
************************************************************************
      subroutine ProcessHard_EW_1spr
     &     (nrdm,ncall1,itmx1,itmx2,hard,hard_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,pid,nrdm
      real*8 sq,hard_sd
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      common /s/ s
      common /pid/ pid
      external function_ew_1spr

      sq = sqrt(s)

      xl(1) = mf**2
      xu(1) = s-2d0*sq*omega
      
      dummy = ranini(nrdm)
      ncall = ncall1
      itmx = itmx1
      call vegas (1, function_ew_1spr, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (1, function_ew_1spr, avgi, sd, chi2a)
      hard = avgi
      hard_sd = sd

      return
      end
************************************************************************
      subroutine ProcessHard_QCD_1spr
     &     (nrdm,ncall1,itmx1,itmx2,hard,hard_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,pid,nrdm
      real*8 sq,hard_sd
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      common /s/ s
      common /pid/ pid
      external function_qcd_1spr

      sq = sqrt(s)

      xl(1) = mf**2
      xu(1) = s-2d0*sq*omega

      dummy = ranini(nrdm)
      ncall = ncall1
      itmx = itmx1
      call vegas (1, function_qcd_1spr, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (1, function_qcd_1spr, avgi, sd, chi2a)
      hard = avgi
      hard_sd = sd

      return
      end
************************************************************************
      subroutine ProcessHard_EW (nrdm,ncall1,itmx1,itmx2,hard,hard_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,pid,nrdm
      real*8 sq,hard_sd
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      common /s/ s
      common /pid/ pid
      external function_ew_4d

      sq = sqrt(s)

      xl(1) = mf**2
      xu(1) = s-2d0*sq*omega
      xl(2) = 0d0
      xu(2) = 2d0*pi
      xl(3) = -1d0
      xu(3) = +1d0
      xl(4) = -1d0
      xu(4) = +1d0
      
      dummy = ranini(nrdm)
      ncall = ncall1
      itmx = itmx1
      call vegas (4, function_ew_4d, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (4, function_ew_4d, avgi, sd, chi2a)
      hard = avgi
      hard_sd = sd

      return
      end
************************************************************************
      subroutine ProcessHard_QCD (nrdm,ncall1,itmx1,itmx2,hard,hard_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,pid,nrdm
      real*8 sq,hard_sd
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      common /s/ s
      common /pid/ pid
      external function_qcd_4d

      sq = sqrt(s)

      xl(1) = mf**2
      xu(1) = s-2d0*sq*omega
      xl(2) = 0d0
      xu(2) = 2d0*pi
      xl(3) = -1d0
      xu(3) = +1d0
      xl(4) = -1d0
      xu(4) = +1d0
      
      dummy = ranini(nrdm)
      ncall = ncall1
      itmx = itmx1
      call vegas (4, function_qcd_4d, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (4, function_qcd_4d, avgi, sd, chi2a)
      hard = avgi
      hard_sd = sd

      return
      end
************************************************************************
      real*8 function function_ew_1c (x,wgt)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 x(10),wgt
      common /s/ s
      common /pid/ pid

      cosf = x(1)

      if     ((pid.eq.1).or.(pid.eq.2)) then
         t = - s/2*(1+cosf)
         u = - s/2*(1-cosf)
      elseif ((pid.eq.3).or.(pid.eq.4).or.(pid.eq.5).or.(pid.eq.6)) then
         t = - (s-mf**2)/2*(1+cosf)
         u = - (s-mf**2)/2*(1-cosf)
      endif

      function_ew_1c = 0d0
      if (iborn.eq.1) then
         if     (pid.eq.1) then
            call cc_br_1314_1112 (-s,-t,-u,born,soft,hard)
         elseif (pid.eq.2) then
            call cc_br_1413_1211 (-s,-t,-u,born,soft,hard)
         elseif (pid.eq.3) then
            call cc_br_1314_1516 (-s,-t,-u,born,soft,hard)
         elseif (pid.eq.4) then
            call cc_br_1413_1615 (-s,-t,-u,born,soft,hard)
         elseif (pid.eq.5) then
            call cc_br_1314_1920 (-s,-t,-u,born,soft,hard)
         elseif (pid.eq.6) then
            call cc_br_1413_2019 (-s,-t,-u,born,soft,hard)
         endif
         function_ew_1c = born
         return
      elseif (iborn.eq.0) then
         sigma = 0d0
         if     (pid.eq.1) then
            call cc_br_1314_1112 (-s,-t,-u,born,soft,hard)
         elseif (pid.eq.2) then
            call cc_br_1413_1211 (-s,-t,-u,born,soft,hard)
         elseif (pid.eq.3) then
            call cc_br_1314_1516 (-s,-t,-u,born,soft,hard)
         elseif (pid.eq.4) then
            call cc_br_1413_1615 (-s,-t,-u,born,soft,hard)
         elseif (pid.eq.5) then
            call cc_br_1314_1920 (-s,-t,-u,born,soft,hard)
         elseif (pid.eq.6) then
            call cc_br_1413_2019 (-s,-t,-u,born,soft,hard)
         endif
         if (iew.eq.1) then
            if     (pid.eq.1) then
               call cc_si_1314_1112 (s,t,u,sigma)
            elseif (pid.eq.2) then
               call cc_si_1413_1211 (s,t,u,sigma)
            elseif (pid.eq.3) then
               call cc_si_1314_1516 (s,t,u,sigma)
            elseif (pid.eq.4) then         
               call cc_si_1413_1615 (s,t,u,sigma)
            elseif (pid.eq.5) then
               call cc_si_1314_1920 (s,t,u,sigma)
            elseif (pid.eq.6) then         
               call cc_si_1413_2019 (s,t,u,sigma)
            endif
            sigma = sigma - born
         endif
         if (iqed.ge.1) then
            sigma = sigma + soft
         endif
         function_ew_1c = sigma
         return
      else
         print *, "wrong flags values = ", iborn
      endif

      return
      end
************************************************************************
      real*8 function function_qcd_1c (x,wgt)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 x(10),wgt
      common /s/ s
      common /pid/ pid

      cosf = x(1)

      if     ((pid.eq.1).or.(pid.eq.2)) then
         t = - s/2*(1-cosf)
         u = - s/2*(1+cosf)
      elseif ((pid.eq.3).or.(pid.eq.4).or.(pid.eq.5).or.(pid.eq.6)) then
         t = - (s-mf**2)/2*(1+cosf)
         u = - (s-mf**2)/2*(1-cosf)
      endif

      function_qcd_1c = 0d0
      if     (pid.eq.1) then
         call cc_qcd_br_1314_1112 (-s,-t,-u,born,virt,hard)
      elseif (pid.eq.2) then
         call cc_qcd_br_1413_1211 (-s,-t,-u,born,virt,hard)
      elseif (pid.eq.3) then
         call cc_qcd_br_1314_1516 (-s,-t,-u,born,virt,hard)
      elseif (pid.eq.4) then
         call cc_qcd_br_1413_1615 (-s,-t,-u,born,virt,hard)
      elseif (pid.eq.5) then
         call cc_qcd_br_1314_1920 (-s,-t,-u,born,virt,hard)
      elseif (pid.eq.6) then
         call cc_qcd_br_1413_2019 (-s,-t,-u,born,virt,hard)
      endif
      function_qcd_1c = virt
      return

      return
      end
************************************************************************
      real*8 function function_ew_1s (x,wgt)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 Mfscale,ksi,bornk,brdq
      real*8 x(10),wgt
      common /s/ s
      common /Mfscale/ Mfscale
      common /pid/ pid

      ksi = x(1)

      function_ew_1s = 0d0
      if     (pid.eq.1) then
         call cc_bo_1314_1112 (s*ksi,bornk)
      elseif (pid.eq.2) then
         call cc_bo_1413_1211 (s*ksi,bornk)
      elseif (pid.eq.3) then
         call cc_bo_1314_1516 (s*ksi,bornk)
      elseif (pid.eq.4) then
         call cc_bo_1413_1615 (s*ksi,bornk)
      elseif (pid.eq.5) then
         call cc_bo_1314_1920 (s*ksi,bornk)
      elseif (pid.eq.6) then
         call cc_bo_1413_2019 (s*ksi,bornk)
      endif

*     (EW)
*     see Eq.(12) DYCC
         brdq =
     &        alpha/2/pi*qup**2*(
     &        (1d0+ksi**2)/(1d0-ksi)*(log(Mfscale**2/mup**2)
     &        -2d0*log(1d0-ksi)-1d0))
     &        *(bornk)
     &        +alpha/2/pi*qdn**2*(
     &        (1d0+ksi**2)/(1d0-ksi)*(log(Mfscale**2/mdn**2)
     &        -2d0*log(1d0-ksi)-1d0))
     &        *(bornk)
      function_ew_1s = brdq
      
      return
      end
************************************************************************
      real*8 function function_qcd_1s (x,wgt)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 Mfscale,ksi,bornk,brdq
      real*8 x(10),wgt
      common /s/ s
      common /Mfscale/ Mfscale
      common /pid/ pid

      ksi = x(1)

      function_qcd_1s = 0d0
      if     (pid.eq.1) then
         call cc_bo_1314_1112 (s*ksi,bornk)
      elseif (pid.eq.2) then
         call cc_bo_1413_1211 (s*ksi,bornk)
      elseif (pid.eq.3) then
         call cc_bo_1314_1516 (s*ksi,bornk)
      elseif (pid.eq.4) then
         call cc_bo_1413_1615 (s*ksi,bornk)
      elseif (pid.eq.5) then
         call cc_bo_1314_1920 (s*ksi,bornk)
      elseif (pid.eq.6) then
         call cc_bo_1413_2019 (s*ksi,bornk)
      endif

*     (QCD)
         brdq =
     &        alphas/2/pi*cf*(
     &        (1d0+ksi**2)/(1d0-ksi)*(log(Mfscale**2/mup**2)
     &        -2d0*log(1d0-ksi)-1d0))
     &        *(bornk)
     &        +alphas/2/pi*cf*(
     &        (1d0+ksi**2)/(1d0-ksi)*(log(Mfscale**2/mdn**2)
     &        -2d0*log(1d0-ksi)-1d0))
     &        *(bornk)
      function_qcd_1s = brdq
      
      return
      end
************************************************************************
      real*8 function function_ew_1spr (x,wgt)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 x(10),wgt
      common /s/ s
      common /pid/ pid

      spr = x(1)

      function_ew_1spr = 0d0
      if     (pid.eq.1) then
         call cc_ha_1314_1112_1spr (s,spr,hard)
      elseif (pid.eq.2) then
         call cc_ha_1413_1211_1spr (s,spr,hard)
      elseif (pid.eq.3) then
         call cc_ha_1314_1516_1spr (s,spr,hard)
      elseif (pid.eq.4) then
         call cc_ha_1413_1615_1spr (s,spr,hard)
      elseif (pid.eq.5) then
         call cc_ha_1314_1920_1spr (s,spr,hard)
      elseif (pid.eq.6) then
         call cc_ha_1413_2019_1spr (s,spr,hard)
      endif
      function_ew_1spr = hard
      
      return
      end
************************************************************************
      real*8 function function_qcd_1spr (x,wgt)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 x(10),wgt
      common /pid/ pid
      common /s/ s

      spr = x(1)

      function_qcd_1spr = 0d0
      if     (pid.eq.1) then
         call cc_qcd_ha_1314_1112_1spr (s,spr,hard)
      elseif (pid.eq.2) then
         call cc_qcd_ha_1413_1211_1spr (s,spr,hard)
      elseif (pid.eq.3) then
         call cc_qcd_ha_1314_1516_1spr (s,spr,hard)
      elseif (pid.eq.4) then
         call cc_qcd_ha_1413_1615_1spr (s,spr,hard)
      elseif (pid.eq.5) then
         call cc_qcd_ha_1314_1920_1spr (s,spr,hard)
      elseif (pid.eq.6) then
         call cc_qcd_ha_1413_2019_1spr (s,spr,hard)
      endif
      function_qcd_1spr = hard
      
      return
      end
************************************************************************
      real*8 function function_ew_4d (x,wgt)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 x(10),wgt,ph3,cth3,cth5
      common /pid/ pid
      common /s/ s

      spr  = x(1)
      ph3  = x(2)
      cth3 = x(3)
      cth5 = x(4)

      function_ew_4d = 0d0
      if     (pid.eq.1) then
         call cc_ha_1314_1112 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.2) then
         call cc_ha_1413_1211 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.3) then
         call cc_ha_1314_1516 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.4) then
         call cc_ha_1413_1615 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.5) then
         call cc_ha_1314_1920 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.6) then
         call cc_ha_1413_2019 (s,spr,ph3,cth3,cth5,hard)
      endif
      function_ew_4d = hard

      return
      end
************************************************************************
      real*8 function function_qcd_4d (x,wgt)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 x(10),wgt,ph3,cth3,cth5
      common /pid/ pid
      common /s/ s

      spr  = x(1)
      ph3  = x(2)
      cth3 = x(3)
      cth5 = x(4)

      function_qcd_4d = 0d0
      if     (pid.eq.1) then
         call cc_qcd_ha_1314_1112 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.2) then
         call cc_qcd_ha_1413_1211 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.3) then
         call cc_qcd_ha_1314_1516 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.4) then
         call cc_qcd_ha_1413_1615 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.5) then
         call cc_qcd_ha_1314_1920 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.6) then
         call cc_qcd_ha_1413_2019 (s,spr,ph3,cth3,cth5,hard)
      endif
      function_qcd_4d = hard

      return
      end
************************************************************************
      subroutine ProcessInit (pid)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 Mfscale
      common /Mfscale/ Mfscale

      if ((pid.ne.1).and.(pid.ne.2).and.(pid.ne.3).and.(pid.ne.4).and.
     &     (pid.ne.5).and.(pid.ne.6)) call PrintMessage (1,pid)

      print *
      call PrintProcess (pid)
      print *

      if     (pid.eq.1) then
         print *, "mup(GeV) =", mup
         print *, "mdn(GeV) =", mdn
         print *, "mel(GeV) =", mel
         Mfscale = mw
         mf = mel
         nc = 1d0
         fc = 3d0
      elseif (pid.eq.2) then
         print *, "mup(GeV) =", mup
         print *, "mdn(GeV) =", mdn
         print *, "mel(GeV) =", mel
         Mfscale = mw
         mf = mel
         nc = 1d0
         fc = 3d0
      elseif (pid.eq.3) then
         print *, "mup(GeV) =", mup
         print *, "mdn(GeV) =", mdn
         print *, "mmo(GeV) =", mmo
         Mfscale = mw
         mf = mmo
         nc = 1d0
         fc = 3d0
      elseif (pid.eq.4) then
         print *, "mup(GeV) =", mup
         print *, "mdn(GeV) =", mdn
         print *, "mmo(GeV) =", mmo
         Mfscale = mw
         mf = mmo
         nc = 1d0
         fc = 3d0
      elseif (pid.eq.5) then
         print *, "mup(GeV) =", mup
         print *, "mdn(GeV) =", mdn
         print *, "mta(GeV) =", mta
         Mfscale = mw
         mf = mta
         nc = 1d0
         fc = 3d0
      elseif (pid.eq.6) then
         print *, "mup(GeV) =", mup
         print *, "mdn(GeV) =", mdn
         print *, "mta(GeV) =", mta
         Mfscale = mw
         mf = mta
         nc = 1d0
         fc = 3d0
      endif

      return
      end
************************************************************************
      subroutine BetaInit (pid)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 sq
      common /s/ s

      sq = sqrt(s)

      if     ((pid.eq.1).or.(pid.eq.2)) then
         if (sq.le.mf) call PrintMessage (2,pid)
         betaf = 1d0
      elseif ((pid.eq.3).or.(pid.eq.4).or.(pid.eq.5).or.(pid.eq.6)) then
         if (sq.le.mf) call PrintMessage (2,pid)
         betaf = (s-mf**2)/(s+mf**2)
      endif

      return
      end
************************************************************************
      subroutine PrintMessage (pmg,pid)
      implicit none!
#include "s2n_declare.h"
      integer*4 pmg,pid
      real*8 sq0
      common /s/ s

      sq0 = mf

      if (pmg.eq.1)
     &     print *, "Fatal error, invalid process number: pid = ", pid
      if (pmg.eq.2)
     &     print *, "Fatal error, process under threshold: sq = ",
     &     sqrt(s),"  <  (Mf1+Mf2) = ", sq0 

      stop
      end
************************************************************************
      subroutine PrintConsts (pmg)
      implicit none!
      integer pmg
#include "s2n_declare.h"

      if ((pmg.eq.0).or.(pmg.eq.1)) then
         print *
         print *, "flag iqed     = ",iqed
         print *, "flag iew      = ",iew
         print *, "flag qcd      = ",iqcd
         print *, "flag gfscheme = ",gfscheme
         print *, "flag ilin     = ",ilin
      endif
      if ((pmg.eq.0).or.(pmg.eq.2)) then
         print *
         print *, "alphai    = ",alphai
         print *, "alpha     = ",alpha
         print *, "GF(GeV-^2)= ",gf
         print *, "g         = ",g
         print *, "e         = ",e
         print *, "alphas    = ",alpha
         print *, "gs        = ",gs
         print *, "cwm       = ",ctw
         print *, "swm2      = ",stw2
         print *, "mh(GeV)   = ",mh
         print *, "mw(GeV)   = ",mw
         print *, "mz(GeV)   = ",mz
         print *, "wz(GeV)   = ",wz
         print *, "ww(GeV)   = ",ww
         print *, "wh(GeV)   = ",wh
         print *, "wtp(GeV)  = ",wtp
      endif

      return
      end
************************************************************************
      subroutine PrintProcess (pid)
      implicit none!
      integer*4 pid

      if (pid.eq.1) print *,
     &  "pid = ", pid, ": proccess anti-up + dn -> en   + el^-"
      if (pid.eq.2) print *,
     &  "pid = ", pid, ": proccess anti-dn + up -> el^+ + en"
      if (pid.eq.3) print *,
     &  "pid = ", pid, ": proccess anti-up + dn -> mn   + mo^-"
      if (pid.eq.4) print *,
     &  "pid = ", pid, ": proccess anti-dn + up -> mo^+ + mn"
      if (pid.eq.5) print *,
     &  "pid = ", pid, ": proccess anti-up + dn -> tn   + ta^-"
      if (pid.eq.6) print *,
     &  "pid = ", pid, ": proccess anti-dn + up -> ta^+ + tn"

      return
      end
************************************************************************
